//Image Display Direct Draw
//Compiled on Pelles C by Rich Berg
//Displays sample Binary Images 
// INCLUDES ///////////////////////////////////////////////
#define WIN32_LEAN_AND_MEAN

// You need to tell DirectInput if you're using v7.0 or less
#define DIRECTINPUT_VERSION 0x0700

#define INITGUID

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h> 
#include <ddraw.h>
#include <dinput.h>
// DEFINES ////////////////////////////////////////////////

// defines for windows 
#define WINDOW_CLASS_NAME "WINCLASS1"

// default screen size
#define SCREEN_WIDTH    640  // size of screen
#define SCREEN_HEIGHT   480
#define SCREEN_BPP      8    // bits per pixel
#define MAX_COLORS      256  // maximum colors

// TYPES //////////////////////////////////////////////////////

// basic unsigned types
typedef unsigned short USHORT;
typedef unsigned short WORD;
typedef unsigned char  UCHAR;
typedef unsigned char  BYTE;

// MACROS /////////////////////////////////////////////////
#define KEYDOWN(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 1 : 0)
#define KEYUP(vk_code)   ((GetAsyncKeyState(vk_code) & 0x8000) ? 0 : 1)

// initializes a direct draw struct
#define DDRAW_INIT_STRUCT(ddstruct) { memset(&ddstruct,0,sizeof(ddstruct)); ddstruct.dwSize=sizeof(ddstruct); }

// GLOBALS ////////////////////////////////////////////////
HWND      main_window_handle = NULL; // globally track main window
HINSTANCE hinstance_app      = NULL; // globally track hinstance
int window_closed      = 0;    // tracks if window is closed
char buffer[80];                // used to print text
// directinput globals
LPDIRECTINPUT7        lpdi      = NULL;    // dinput object
LPDIRECTINPUTDEVICE7  lpdikey   = NULL;    // dinput keyboard
LPDIRECTINPUTDEVICE7  lpdimouse = NULL;    // dinput mouse
UCHAR        keystate[256]; // contains keyboard state table
DIMOUSESTATE mouse_state;         // contains state of mouse
//Directdraw stuff
LPDIRECTDRAW          lpdd         = NULL;   // dd object
LPDIRECTDRAW7         lpdd7        = NULL;   // dd7 object
LPDIRECTDRAWSURFACE7  lpddsprimary = NULL;   // dd primary surface
LPDIRECTDRAWSURFACE7  lpddsback    = NULL;   // dd back surface
LPDIRECTDRAWPALETTE   lpddpal      = NULL;   // a pointer to the created dd palette
PALETTEENTRY          palette[256];          // color palette
PALETTEENTRY          save_palette[256];     // used to save palettes
DDSURFACEDESC2        ddsd;                  // a direct draw surface description struct
DDBLTFX               ddbltfx;               // used to fill
DDSCAPS2              ddscaps;               // a direct draw surface capabilities struct

int mouse_x,                    // used to track mouse
    mouse_y;
    
UCHAR mouse_color=15;          // color of mouse brush
int dx;
int dy;
// these are overwritten globally by DD_Init()
int screen_width  = SCREEN_WIDTH,            // width of screen
    screen_height = SCREEN_HEIGHT,           // height of screen
    screen_bpp    = SCREEN_BPP;              // bits per pixel

//PROTOTYPES///////////////////////
int Game_Init(void);
int Game_Main(void);
int Game_Shutdown(void);
void Plot8(int x, int y, UCHAR color, UCHAR *buffer, int mempitch);
void DDraw_Fill_Surface(LPDIRECTDRAWSURFACE7 lpdds,int color);
int Load_Ports(int pixnum, int px, int py);
int Draw_Text_GDI_T(char *text, int x, int y, COLORREF color, LPDIRECTDRAWSURFACE7 lpdds);
int Draw_Text_GDI(char *text, int x, int y, COLORREF color, LPDIRECTDRAWSURFACE7 lpdds); 
int Load_Button(int pixnum, int px, int py);
void Rect_Erase(int section);
int Waiting (void);
int Set_Palette_Entry(int color_index, LPPALETTEENTRY color);
void Main_Pix(void);
void Kwscr_Pix(int pixnum, int px, int py);
void Kwscr2_Pix(int pixnum, int px, int py);
void Load_Battlemap(int *battlepixnum, int season);
void Get_pxpy(int *px, int *py);
void Place_Tiles(int pixnum, int pr, int a);
void Load_WarPieces(int pixnum, int px, int py);
void Load_Borders(int pixnum, int px, int py);

// FUNCTIONS //////////////////////////////////////////////

LRESULT CALLBACK WindowProc(HWND hwnd, 
						    UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
	// this is the main message handler of the system
	PAINTSTRUCT		ps;		// used in WM_PAINT
	// what is the message 
	switch(msg)
		{	
		case WM_CREATE: 
			{
			// do initialization stuff here
			// return success
			return(0);
			} break;

			case WM_PAINT: 
			{
			// simply validate the window 
        		BeginPaint(hwnd,&ps);	 
			// end painting
			EndPaint(hwnd,&ps);

			// return success
			return(0);
   			} break;

		case WM_DESTROY: 
			{

			// kill the application, this sends a WM_QUIT message 
			PostQuitMessage(0);

			// return success
			return(0);
			} break;

		default:break;

		} // end switch

	// process any messages that we didn't take care of 
	return (DefWindowProc(hwnd, msg, wparam, lparam));

} // end WinProc
//////////////////////////////////////////////
int Game_Main(void)
{
	MSG msg;
	int t;
	int season; //0=spring, 1=summer, 2=fall, 3=winter
	PALETTEENTRY blue;
	PALETTEENTRY green;
	PALETTEENTRY red;
	int battlepixnum[85] =  {0, 1, 2, 2, 2, 2, 3, 3, 1, 2, 2, 2, 2, 3, 3,
						       1, 2, 7, 2, 3, 8, 6, 1, 1, 7, 2, 6, 8, 6,
						       1, 1, 7, 2, 3, 8, 6, 2, 1, 8, 2, 3, 8, 6,
						       2, 8, 8, 3, 4, 8, 2, 2, 2, 8, 3, 6, 5, 6,
						       2, 8, 5, 3, 6, 6, 2, 6, 3, 8, 5, 3, 5, 2,
						       6, 3, 2, 5, 6, 5, 2, 6, 3, 2, 2, 3, 6, 2};

	for(t=1; t<15; t++)
	{
		// for now test if user is hitting ESC and send WM_CLOSE
		if (KEYDOWN(VK_ESCAPE))
		{
		 	PostMessage(main_window_handle,WM_CLOSE,0,0);
			window_closed = 1;
		}
		PeekMessage(&msg,NULL,0,0,PM_REMOVE)	;	
		{ 
			// translate any accelerator keys
			TranslateMessage(&msg);
			// send the message to the window proc
			DispatchMessage(&msg);
		} // end if  
	}
	ShowCursor(FALSE);
	Draw_Text_GDI("Image Display", 260, 20, RGB(255, 255, 255), lpddsprimary);
	Load_Ports(0, 40, 50);
	Load_Ports(1, 160, 50);
	Load_Ports(3, 280, 50);
	Load_Ports(5, 400, 50);
	Load_Ports(6, 520, 50);
	Draw_Text_GDI("Valiant", 60, 120, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Angel", 185, 120, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Merlin", 300, 120, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Death Watch", 400, 120, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Golden Sun", 525, 120, RGB(255, 255, 255), lpddsprimary);
	Load_Ports(7, 40, 140);
	Load_Ports(8, 160, 140);
	Load_Ports(9, 280, 140);
	Load_Ports(10, 400, 140);
	Load_Ports(12, 520, 140);
	Draw_Text_GDI("The Slasher", 40, 210, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Marauder", 170, 210, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Fast Rider", 285, 210, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Dragon of Zil", 395, 210, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Chain Lightening", 510, 210, RGB(255, 255, 255), lpddsprimary);
	Load_Ports(15, 40, 230);
	Load_Ports(16, 160, 230);
	Load_Ports(17, 280, 230);
	Load_Ports(18, 400, 230);
	Load_Ports(19, 520, 230);
	Draw_Text_GDI("Widow Maker", 40, 300, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("The Rat", 175, 300, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Shield of Wan", 275, 300, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Night Stalker", 395, 300, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Hedgehog", 530, 300, RGB(255, 255, 255), lpddsprimary);

	Draw_Text_GDI("Some sample portraits of my game participants.", 50, 380, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("These are all binary images - even the button.", 50, 410, RGB(255, 255, 255), lpddsprimary);
	Load_Button(0, 540, 435);
	Waiting();
	Rect_Erase(1);
	Rect_Erase(2);
	ShowCursor(FALSE);
	Load_Ports(4, 270, 60);
	Draw_Text_GDI("Red Sun", 285, 130, RGB(255, 0, 0), lpddsprimary);
	Sleep(2000);
	blue.peRed = 0;
	blue.peGreen = 0;
	blue.peBlue = 255;
	blue.peFlags = PC_NOCOLLAPSE;
	Set_Palette_Entry(4, &blue);
	Draw_Text_GDI("Change a palette entry and he's Blue Sun", 50, 380, RGB(255, 255, 255), lpddsprimary);
	Sleep(2000);
	green.peRed = 0;
	green.peGreen = 255;
	green.peBlue = 0;
	green.peFlags = PC_NOCOLLAPSE;
	Set_Palette_Entry(4, &green);
	Draw_Text_GDI("or Green Sun", 50, 400, RGB(255, 255, 255), lpddsprimary);
	Sleep(2000);
	red.peRed = 255;
	red.peGreen = 0;
	red.peBlue = 0;
	red.peFlags = PC_NOCOLLAPSE;
	Set_Palette_Entry(4, &red);
	Draw_Text_GDI("Back to normal.", 50, 420, RGB(255, 255, 255), lpddsprimary);
	Load_Button(0, 540, 435);
	Waiting();
	Rect_Erase(1);
	Rect_Erase(2);
	ShowCursor(FALSE);
	lpddsback->lpVtbl->Lock(lpddsback, NULL, &ddsd,DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT,NULL);
	Main_Pix();
	lpddsback->lpVtbl->Unlock(lpddsback, NULL);
	while (FAILED(lpddsprimary->lpVtbl->Flip(lpddsprimary, NULL, DDFLIP_WAIT)));
	Draw_Text_GDI("Image Display", 260, 20, RGB(255, 255, 255), lpddsprimary);
	Load_Button(0, 540, 435);
	Draw_Text_GDI("Gold", 140, 119, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Food", 140, 155, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Metal", 140, 190, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Furs", 140, 226, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Rate", 140, 263, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Heroes", 280, 119, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Men", 280, 155, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Arms", 280, 190, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Skill", 280, 226, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("People", 280, 263, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Castles", 418, 119, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Support", 418, 155, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Flood", 418, 190, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Farm", 418, 226, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Wealth", 418, 263, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Images for game stats.", 50, 380, RGB(255, 255, 255), lpddsprimary);
	Waiting();
	Rect_Erase(1);
	Rect_Erase(2);
	ShowCursor(FALSE);
	season = 1;
	Load_Battlemap(battlepixnum, season);
	Draw_Text_GDI("Image Display", 260, 20, RGB(255, 255, 255), lpddsprimary);
	Load_Button(0, 540, 435);
	Draw_Text_GDI("Part of a battle map made with tiles created", 50, 380, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("with the binary image drawing program.", 50, 400, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Summer version", 50, 420, RGB(255, 255, 255), lpddsprimary);
	Waiting();
	Rect_Erase(1);
	Rect_Erase(2);
	ShowCursor(FALSE);
	season = 2;
	Load_Battlemap(battlepixnum, season);
	Draw_Text_GDI("Image Display", 260, 20, RGB(255, 255, 255), lpddsprimary);
	Load_Button(0, 540, 435);
	Draw_Text_GDI("Fall version", 50, 380, RGB(255, 255, 255), lpddsprimary);
	Waiting();
	Rect_Erase(1);
	Rect_Erase(2);
	ShowCursor(FALSE);
	season = 3;
	Load_Battlemap(battlepixnum, season);
	Draw_Text_GDI("Image Display", 260, 20, RGB(255, 255, 255), lpddsprimary);
	Load_Button(0, 540, 435);
	Draw_Text_GDI("Winter version", 50, 380, RGB(255, 255, 255), lpddsprimary);
	Waiting();
	Rect_Erase(1);
	Rect_Erase(2);
	ShowCursor(FALSE);
	season = 0;
	Load_Battlemap(battlepixnum, season);
	Draw_Text_GDI("Image Display", 260, 20, RGB(255, 255, 255), lpddsprimary);
	Load_Button(0, 540, 435);
	Draw_Text_GDI("Spring version", 50, 380, RGB(255, 255, 255), lpddsprimary);
	Sleep(2000);
	Draw_Text_GDI("Add some men", 50, 400, RGB(255, 255, 255), lpddsprimary);
	Place_Tiles(3, 39, 1);
	Draw_Text_GDI_T("100", dx, dy, RGB(255, 255, 255), lpddsprimary);
	Place_Tiles(0, 32, 1);
	Draw_Text_GDI_T("94", dx, dy, RGB(255, 255, 255), lpddsprimary);
	Place_Tiles(0, 53, 1);
	Draw_Text_GDI_T("61", dx, dy, RGB(255, 255, 255), lpddsprimary);
	Place_Tiles(4, 60, 1);
	Draw_Text_GDI_T("98", dx, dy, RGB(255, 255, 255), lpddsprimary);
	Place_Tiles(2, 67, 1);
	Draw_Text_GDI_T("55", dx, dy, RGB(255, 255, 255), lpddsprimary);
	Place_Tiles(1, 54, 1);
	Draw_Text_GDI_T("100", dx, dy, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Whose turn is it?", 50, 420, RGB(255, 255, 255), lpddsprimary);
	Sleep(2000);
	Place_Tiles(1, 54, 2);
	Sleep(500);
	Place_Tiles(0, 54, 2);
	Sleep(500);
	Place_Tiles(1, 54, 2);
	Sleep(500);
	Place_Tiles(0, 54, 2);
	Place_Tiles(1, 54, 2);
	Waiting();
	Rect_Erase(1);
	Rect_Erase(2);
	ShowCursor(FALSE);
	Draw_Text_GDI("That's it folks.", 50, 380, RGB(255, 255, 255), lpddsprimary);
	Draw_Text_GDI("Thanks for watching.", 50, 400, RGB(255, 255, 255), lpddsprimary);
	Waiting();
	return(1);
}
//////////////////////////////
void Load_Borders(int pixnum, int px, int py)
{  //This loads the appropriate tile border
	int pixx, pixy;
	CHAR pixc;
	FILE *fptr;
	long int pixoffset;
	long int size = 1344;

	lpddsprimary->lpVtbl->Lock(lpddsprimary, NULL, &ddsd,
                   DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT,
                   NULL);
	if((fptr=fopen("Borders.pix","rb"))==NULL) /*Load image*/
	{Draw_Text_GDI("Load Failure", 30, 430, RGB(255, 255, 255),lpddsback);}
	
	pixoffset = (pixnum) * size;
	fseek(fptr, pixoffset, SEEK_SET);
	for(pixy = py; pixy<(py+32); pixy++)
	{
		for(pixx=px; pixx<(px+42); pixx++)
		{ 
			pixc = getc(fptr);
			// plot the pixel
			if(pixc!=0)
				Plot8(pixx, pixy, pixc, (UCHAR *)ddsd.lpSurface, (int)ddsd.lPitch);        
		}
	}
    	fclose(fptr);
	lpddsprimary->lpVtbl->Unlock(lpddsprimary, NULL);
}
/////////////////////////////
void Place_Tiles(int pixnum, int pr, int a)
{
	int p, r, px, py;
	int s[15] = {0, 67, 151, 235, 319, 403, 487, 571, 25, 109, 193, 277, 361, 445, 529};
	int t[15] = {0, 42, 42, 42, 42, 42, 42, 42, 58, 58, 58, 58, 58, 58, 58};  //added 40

	if(pr<15)
		px = s[pr];
	if(pr>14)
	{
		r = pr%14;
		if(r==0)
			r = 14;
		px = s[r];
	}
	if(pr<15)
		py = t[pr];
	if(pr>14)
	{
		p = pr/14;
		r = pr%14;
		if(r==0)
		{
			p--;
			r = 14;
		}
		py = (p*32)+t[r];
	}
	dx = px+9;
	dy = py+14;
	if(a==1)
		Load_WarPieces(pixnum, px, py);
	if(a==2)
		Load_Borders(pixnum, px, py);
}
/////////////////////////////
void Load_WarPieces(int pixnum, int px, int py)
{  //This loads the actual war pieces (men)
	int pixx, pixy;
	CHAR pixc;
	FILE *fptr;
	long int pixoffset;
	long int size = 1344;
	
	lpddsprimary->lpVtbl->Lock(lpddsprimary, NULL, &ddsd,
                   DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT,
                   NULL);
	if((fptr=fopen("Pieces.pix","rb"))==NULL) /*Load image*/
	{Draw_Text_GDI("Load Failure", 30, 430, RGB(255, 255, 255),lpddsback);}
	
	pixoffset = (pixnum) * size;
	fseek(fptr, pixoffset, SEEK_SET);
	for(pixy = py; pixy<(py+32); pixy++)
	{
		for(pixx=px; pixx<(px+42); pixx++)
		{ 
			pixc = getc(fptr);
			// plot the pixel
			Plot8(pixx, pixy, pixc, (UCHAR *)ddsd.lpSurface, (int)ddsd.lPitch);        
		}
	}
    	fclose(fptr);
	lpddsprimary->lpVtbl->Unlock(lpddsprimary, NULL);
}
/////////////////////////////
void Load_Battlemap(int *battlepixnum, int season)
{
	int a;
	int px[85];
	int py[85];
	long int size = 1344;
	long int pixoffset;
	int pixx, pixy, pixc;
	FILE *fptr;

	Get_pxpy(px, py);
	lpddsback->lpVtbl->Lock(lpddsback, NULL, &ddsd,DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT,NULL);

	if((fptr=fopen("WarTiles.pix","rb"))==NULL) // loads the terrain tiles
	{Draw_Text_GDI("Load Failure", 30, 430, RGB(255, 255, 255),lpddsback);}
	for(a=1; a<85; a++)
	{
		pixoffset = (battlepixnum[a]+(season*9)) * size;
		fseek(fptr, pixoffset, 0);
		for(pixy = py[a]; pixy<(py[a]+32); pixy++)
		{
			for(pixx=px[a]; pixx<(px[a]+42); pixx++)
			{ 
				pixc = getc(fptr);
				// plot the pixel
				Plot8(pixx, pixy, pixc, (UCHAR *)ddsd.lpSurface, (int)ddsd.lPitch);        
			}
		}
	}
    fclose(fptr);
	lpddsback->lpVtbl->Unlock(lpddsback, NULL);
	while (FAILED(lpddsprimary->lpVtbl->Flip(lpddsprimary, NULL, DDFLIP_WAIT)));
}
////////////////////////////
void Get_pxpy(int *px, int *py)
{
	int p, r, sqr;
	int s[15] = {0, 67, 151, 235, 319, 403, 487, 571, 25, 109, 193, 277, 361, 445, 529};
	int t[15] = {0, 42, 42, 42, 42, 42, 42, 42, 58, 58, 58, 58, 58, 58, 58};  //adding 40

	px[0] = 0;
	for(sqr=1; sqr<15; sqr++)
		px[sqr] = s[sqr];
	for(sqr=15; sqr<85; sqr++)
	{
		r = sqr%14;
		if(r==0)
			r = 14;
		px[sqr] = s[r];
	}
	py[0] = 0;
	for(sqr=1; sqr<15; sqr++)
		py[sqr] = t[sqr];
	for(sqr=15; sqr<85; sqr++)
	{
		p = sqr/14;
		r = sqr%14;
		if(r==0)
		{
			p--;
			r = 14;
		}
		py[sqr] = (p*32)+t[r];
	}
}
////////////////////////////
void Main_Pix(void)
{
	Kwscr_Pix(1, 104, 112);  //Gold and Food
	Kwscr_Pix(2, 104, 184); //Metal and Furs
	Kwscr2_Pix(3, 104, 256); //Rate
	Kwscr_Pix(4, 242, 112);  //Heroes and Men
	Kwscr_Pix(5, 242, 184); //Arms and Skill
	Kwscr2_Pix(6, 242, 256); //People
	Kwscr_Pix(7, 380, 112);  //Castles and Support
	Kwscr_Pix(8, 380, 184);  //Flood and Farm
	Kwscr2_Pix(9, 380, 256); //Wealth
}
//////////////////////////
void Kwscr_Pix(int pixnum, int px, int py)
{
	int pixx, pixy;
	CHAR pixc;
	FILE *fptr;
	long int pixoffset;
	long int size = 5084;

	if((fptr=fopen("Newkwscr.pix","rb"))==NULL) /*Load image*/
	{Draw_Text_GDI("Load Failure", 30, 430, RGB(255, 255, 255),lpddsback);}
	
	pixoffset = (pixnum-1) * size;
	fseek(fptr, pixoffset, SEEK_SET);
	for(pixy = py; pixy<(py+62); pixy++)
	{
		for(pixx=px; pixx<(px+82); pixx++)
		{ 
			pixc = getc(fptr);
			// plot the pixel
			Plot8(pixx, pixy, pixc, (UCHAR *)ddsd.lpSurface, (int)ddsd.lPitch);        
		}
	}
    fclose(fptr);
}
////////////////////
void Kwscr2_Pix(int pixnum, int px, int py)
{
	int pixx, pixy;
	CHAR pixc;
	FILE *fptr;
	long int pixoffset;
	long int size = 5084;

	if((fptr=fopen("Newkwscr.pix","rb"))==NULL) /*Load image*/
	{Draw_Text_GDI("Load Failure", 30, 430, RGB(255, 255, 255),lpddsback);}
	
	pixoffset = (pixnum-1) * size;
	fseek(fptr, pixoffset, SEEK_SET);
	for(pixy = py; pixy<(py+38); pixy++)
	{
		for(pixx=px; pixx<(px+82); pixx++)
		{ 
			pixc = getc(fptr);
			// plot the pixel
			Plot8(pixx, pixy, pixc, (UCHAR *)ddsd.lpSurface, (int)ddsd.lPitch);        
		}
	}
    fclose(fptr);
}
//////////////////////////////////////////////
int Set_Palette_Entry(int color_index, LPPALETTEENTRY color)
{
	// this function sets a palette color in the palette
	lpddpal->lpVtbl->SetEntries(lpddpal, 0,color_index,1,color);

	// set data in shadow palette
	memcpy(&palette[color_index],color,sizeof(PALETTEENTRY));

	// return success
	return(1);
} // end Set_Palette_Entry

//////////////////////////////////////////////
int Waiting (void)
{
	MSG msg;
	int done = 0;
	RECT button_rect;
	button_rect.top = 435;
	button_rect.left = 540;
	button_rect.bottom = 447;
	button_rect.right = 585;

	mouse_x = 550;
	mouse_y = 441;
	SetCursorPos(mouse_x, mouse_y);
	ClipCursor(&button_rect);
	ShowCursor(TRUE);
	mouse_state.rgbButtons[0] = 0;
	mouse_state.rgbButtons[1] = 0;
	do
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{ 
			// test if this is a quit
			if (msg.message == WM_QUIT)
				break;
			// translate any accelerator keys
			TranslateMessage(&msg);
			// send the message to the window proc
			DispatchMessage(&msg);
		} // end if
		if (KEYDOWN(VK_ESCAPE))
		{
			PostMessage(main_window_handle,WM_CLOSE,0,0);
			window_closed = 1;
		}
		// get the input from the mouse
		lpdimouse->lpVtbl->GetDeviceState(lpdimouse, sizeof(DIMOUSESTATE), (LPVOID)&mouse_state);
		// move the mouse cursor
		mouse_x+=(mouse_state.lX); 
		mouse_y+=(mouse_state.lY); 
		SetCursorPos(mouse_x, mouse_y);
		if(mouse_state.rgbButtons[0] >0 || mouse_state.rgbButtons[1] >0)
			done = 1;
	}
	while (done == 0);
	ClipCursor(NULL);
	mouse_state.rgbButtons[0] = 0;
	mouse_state.rgbButtons[1] = 0;
	Sleep(500);
	
	return(1);
}
//////////////////////////////////////////////
void Rect_Erase(int section)
{
	HDC xdc;

	lpddsprimary->lpVtbl->GetDC(lpddsprimary, &xdc);
	HPEN black_pen = CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
	HPEN oldpen;	
	HBRUSH black_brush = CreateSolidBrush(RGB(0, 0, 0));
	HBRUSH oldbrush;
	SetBkColor(xdc, RGB(0, 0, 0));
	oldpen = (HPEN)SelectObject(xdc, black_pen);
	oldbrush = (HBRUSH)SelectObject(xdc, black_brush);
	if(section == 1)  //Erase the top of the screen
		Rectangle(xdc, 20, 37, 670, 370);
	if(section == 2)  //Erase the print area
		Rectangle(xdc, 50, 370, 520, 470);
	SelectObject(xdc, oldpen);
	SelectObject(xdc, oldbrush);
	DeleteObject(black_pen);
	DeleteObject(black_brush);
	lpddsprimary->lpVtbl->ReleaseDC(lpddsprimary,xdc);
}
//////////////////////////////////////////////
int Load_Ports(int pixnum, int px, int py)
{
	int pixx, pixy;
	int wide = 82;
	int high = 62;
	UCHAR pixc;
	FILE *fptr;
	long int pixoffset;
	long int size = wide * high;
	
	if (FAILED(lpddsprimary->lpVtbl->Lock(lpddsprimary, NULL, &ddsd,
                   DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT,
                   NULL)))
		return(0);
	if((fptr=fopen("ports.pix","a+b"))==NULL) /*Load image*/
	{Draw_Text_GDI("Load Failure", 30, 430, RGB(255, 255, 255),lpddsprimary); exit(1);}
	pixoffset = (pixnum) * size;
	fseek(fptr, pixoffset, SEEK_SET);
	for(pixy = py; pixy<(py + high); pixy++)
	{
		for(pixx=px; pixx<(px + wide); pixx++)
		{ 
			pixc = getc(fptr);
			// plot the pixel
			Plot8(pixx, pixy, pixc, (UCHAR *)ddsd.lpSurface, (int)ddsd.lPitch);        
		}
	}
	fclose(fptr);
   	// now unlock the primarybuffer surface
   	if (FAILED(lpddsprimary->lpVtbl->Unlock(lpddsprimary, NULL)))
		return(0);
	return(1);
}
//////////////////////////////////////////////
int Load_Button(int pixnum, int px, int py)
{
	int pixx, pixy;
	int wide = 45;
	int high = 12;
	UCHAR pixc;
	FILE *fptr;
	long int pixoffset;
	long int size = wide * high;
	
	if (FAILED(lpddsprimary->lpVtbl->Lock(lpddsprimary, NULL, &ddsd,
                   DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT,
                   NULL)))
		return(0);
	if((fptr=fopen("button.pix","a+b"))==NULL) /*Load image*/
	{Draw_Text_GDI("Load Failure", 30, 430, RGB(255, 255, 255),lpddsprimary); exit(1);}
	pixoffset = (pixnum) * size;
	fseek(fptr, pixoffset, SEEK_SET);
	for(pixy = py; pixy<(py + high); pixy++)
	{
		for(pixx=px; pixx<(px + wide); pixx++)
		{ 
			pixc = getc(fptr);
			// plot the pixel
			Plot8(pixx, pixy, pixc, (UCHAR *)ddsd.lpSurface, (int)ddsd.lPitch);        
		}
	}
	fclose(fptr);
   	// now unlock the primarybuffer surface
   	if (FAILED(lpddsprimary->lpVtbl->Unlock(lpddsprimary, NULL)))
		return(0);
	return(1);
}
//////////////////////////////////////////////
void Plot8(int x, int y, UCHAR color, UCHAR *buffer, int mempitch)
{
	buffer[x + y*mempitch] = color;
}
/////////////////////////////////////////////////
int Draw_Text_GDI_T(char *text, int x, int y, COLORREF color, LPDIRECTDRAWSURFACE7 lpdds)
{
	//A GDI Test
	HDC xdc;  //a dc for the direct draw surface
 
	if (FAILED(lpdds->lpVtbl->GetDC(lpdds, &xdc)))
		return(0);
	SetTextColor(xdc, color);
	SetBkColor(xdc, RGB(0, 0, 0));
	SetBkMode(xdc, TRANSPARENT);
	TextOut(xdc, x, y, text, strlen(text));
	lpdds->lpVtbl->ReleaseDC(lpdds, xdc);
	return(1);
}
/////////////////////
int Draw_Text_GDI(char *text, int x, int y, COLORREF color, LPDIRECTDRAWSURFACE7 lpdds)
{
	HDC xdc;  //a dc for the direct draw surface
 
	if (FAILED(lpdds->lpVtbl->GetDC(lpdds, &xdc)))
		return(0);
	SetTextColor(xdc, color);
	SetBkColor(xdc, RGB(0, 0, 0));
	SetBkMode(xdc, OPAQUE);
	TextOut(xdc, x, y, text, strlen(text));
	lpdds->lpVtbl->ReleaseDC(lpdds, xdc);
	return(1);
}
/////////////////
void DDraw_Fill_Surface(LPDIRECTDRAWSURFACE7 lpdds,int color)
{
	DDBLTFX ddbltfx; // this contains the DDBLTFX structure

	// clear out the structure and set the size field 
	DDRAW_INIT_STRUCT(ddbltfx);

	// set the dwfillcolor field to the desired color
	ddbltfx.dwFillColor = color; 

	// ready to blt to surface
	lpdds->lpVtbl->Blt(lpdds, NULL,       // ptr to dest rectangle
           NULL,       // ptr to source surface, NA            
           NULL,       // ptr to source rectangle, NA
           DDBLT_COLORFILL | DDBLT_WAIT,   // fill and wait                   
           &ddbltfx);  // ptr to DDBLTFX structure

	// return success
//	return(0);
} // end DDraw_Fill_Surface
// WINMAIN ////////////////////////////////////////////////
int WINAPI WinMain(	HINSTANCE hinstance,
					HINSTANCE hprevinstance,
					LPSTR lpcmdline,
					int ncmdshow)
{

	WNDCLASSEX winclass; // this will hold the class we create
	HWND	   hwnd;	 // generic window handle
	MSG		   msg;		 // generic message

	// first fill in the window class stucture
	winclass.cbSize         = sizeof(WNDCLASSEX);
	winclass.style			= CS_DBLCLKS | CS_OWNDC | 
	                         CS_HREDRAW | CS_VREDRAW;
	winclass.lpfnWndProc	= WindowProc;
	winclass.cbClsExtra		= 0;
	winclass.cbWndExtra		= 0;
	winclass.hInstance		= hinstance;
	winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
	winclass.hCursor		= LoadCursor(NULL, IDC_ARROW); 
	winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
	winclass.lpszMenuName	= NULL;
	winclass.lpszClassName	= WINDOW_CLASS_NAME;
	winclass.hIconSm        = LoadIcon(NULL, IDI_APPLICATION);

	// save hinstance in global
	hinstance_app = hinstance;

	// register the window class
	if (!RegisterClassEx(&winclass))
		return(0);

	// create the window
	if (!(hwnd = CreateWindowEx(0,                  // extended style
                            WINDOW_CLASS_NAME,     // class
						    "DirectDraw Test for Game", // title
						    WS_POPUP | WS_VISIBLE,  
					 	    0,0,	  // initial x,y
						    SCREEN_WIDTH, SCREEN_HEIGHT, // initial width, height
						    NULL,	  // handle to parent 
						    NULL,	  // handle to menu
						    hinstance,// instance of this application
						    NULL)))	// extra creation parms
	return(0);

	// save main window handle
	main_window_handle = hwnd;

	// initialize game here
	Game_Init();

	Game_Main();

	Game_Shutdown();

	// return to Windows like this
	return(msg.wParam);

} // end WinMain

////////////////////////////////////////////////////////////

int Game_Init(void)
{
	// this is called once after the initial window is created and
	// before the main event loop is entered, do all your initialization
	// here

//added & before IID_IDirectDraw7
	if (FAILED(DirectDrawCreateEx(NULL, (LPVOID *)&lpdd7, &IID_IDirectDraw7, NULL)))
		return(0); //error
	


	if (FAILED(lpdd7->lpVtbl->SetCooperativeLevel(lpdd7, main_window_handle, 
								DDSCL_FULLSCREEN | DDSCL_ALLOWMODEX |
								DDSCL_EXCLUSIVE | DDSCL_ALLOWREBOOT)))
		return(0); //error

	// set display mode to 640x480x8
	if (FAILED(lpdd7->lpVtbl->SetDisplayMode(lpdd7, SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP,0,0)))
		return(0);


	// clear ddsd and set size
	memset(&ddsd,0,sizeof(ddsd)); 
	ddsd.dwSize = sizeof(ddsd);

	// enable valid fields	
	ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;

	ddsd.dwBackBufferCount = 1;

	// request primary surface with one backbuffer
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP |
                             DDSCAPS_COMPLEX;
    

	// create the primary surface
	if (FAILED(lpdd7->lpVtbl->CreateSurface(lpdd7, &ddsd, &lpddsprimary, NULL)))
		return(0);

	// Attach lpddsback to the backbuffer
      ddsd.ddsCaps.dwCaps = DDSCAPS_BACKBUFFER;
	if (FAILED(lpddsprimary->lpVtbl->GetAttachedSurface(lpddsprimary, &ddsd.ddsCaps, &lpddsback)))
		return(0);

	// build up the palette data array
	for (int color=1; color < 255; color++)
		{
		// fill with random RGB values
		palette[color].peRed   = rand()%256;
		palette[color].peGreen = rand()%256;
		palette[color].peBlue  = rand()%256;

		// set flags field to PC_NOCOLLAPSE
		palette[color].peFlags = PC_NOCOLLAPSE;
		} // end for color


	//add my palette colors
	palette[1].peRed   = 0;
	palette[1].peGreen = 0;
	palette[1].peBlue  = 204;	
	palette[1].peFlags = PC_NOCOLLAPSE;

	palette[2].peRed   = 51;
	palette[2].peGreen = 153;
	palette[2].peBlue  = 51;
	palette[2].peFlags = PC_NOCOLLAPSE;

	palette[3].peRed   = 0;
	palette[3].peGreen = 255;
	palette[3].peBlue  = 255;
	palette[3].peFlags = PC_NOCOLLAPSE;

	palette[4].peRed   = 255;
	palette[4].peGreen = 0;
	palette[4].peBlue  = 0;
	palette[4].peFlags = PC_NOCOLLAPSE;

	palette[5].peRed   = 153;
	palette[5].peGreen = 0;
	palette[5].peBlue  = 204;
	palette[5].peFlags = PC_NOCOLLAPSE;

	palette[6].peRed   = 153;
	palette[6].peGreen = 51;
	palette[6].peBlue  = 0;
	palette[6].peFlags = PC_NOCOLLAPSE;

	palette[7].peRed   = 204;
	palette[7].peGreen = 204;
	palette[7].peBlue  = 204;
	palette[7].peFlags = PC_NOCOLLAPSE;

	palette[8].peRed   = 153;
	palette[8].peGreen = 153;
	palette[8].peBlue  = 153;
	palette[8].peFlags = PC_NOCOLLAPSE;

	palette[9].peRed   = 255;
	palette[9].peGreen = 204;
	palette[9].peBlue  = 102;
	palette[9].peFlags = PC_NOCOLLAPSE;

	palette[10].peRed   = 255;
	palette[10].peGreen = 217;
	palette[10].peBlue  = 217;
	palette[10].peFlags = PC_NOCOLLAPSE;

	palette[11].peRed   = 251;
	palette[11].peGreen = 191;
	palette[11].peBlue  = 191;
	palette[11].peFlags = PC_NOCOLLAPSE;

	palette[12].peRed   = 254;
	palette[12].peGreen = 92;
	palette[12].peBlue  = 92;
	palette[12].peFlags = PC_NOCOLLAPSE;

	palette[13].peRed   = 254;
	palette[13].peGreen = 198;
	palette[13].peBlue  = 198;
	palette[13].peFlags = PC_NOCOLLAPSE;

	palette[14].peRed   = 255;
	palette[14].peGreen = 255;
	palette[14].peBlue  = 64;
	palette[14].peFlags = PC_NOCOLLAPSE;

	palette[15].peRed   = 255;
	palette[15].peGreen = 255;
	palette[15].peBlue  = 255;
	palette[15].peFlags = PC_NOCOLLAPSE;

	palette[16].peRed   = 244;
	palette[16].peGreen = 198;
	palette[16].peBlue  = 126;
	palette[16].peFlags = PC_NOCOLLAPSE;
	
	palette[17].peRed   = 216;
	palette[17].peGreen = 212;
	palette[17].peBlue  = 212;
	palette[17].peFlags = PC_NOCOLLAPSE;

	palette[18].peRed   = 0;
	palette[18].peGreen = 255;
	palette[18].peBlue  = 0;
	palette[18].peFlags = PC_NOCOLLAPSE;

	palette[19].peRed   = 255;
	palette[19].peGreen = 64;
	palette[19].peBlue  = 64;
	palette[19].peFlags = PC_NOCOLLAPSE;

	palette[20].peRed   = 128;
	palette[20].peGreen = 128;
	palette[20].peBlue  = 255;
	palette[20].peFlags = PC_NOCOLLAPSE;

	palette[21].peRed   = 64;
	palette[21].peGreen = 64;
	palette[21].peBlue  = 255;
	palette[21].peFlags = PC_NOCOLLAPSE;
	
	palette[22].peRed   = 0;
	palette[22].peGreen = 192;
	palette[22].peBlue  = 0;
	palette[22].peFlags = PC_NOCOLLAPSE;

	palette[23].peRed   = 150;
	palette[23].peGreen = 100;
	palette[23].peBlue  = 14;
	palette[23].peFlags = PC_NOCOLLAPSE;

	palette[24].peRed   = 151;
	palette[24].peGreen = 131;
	palette[24].peBlue  = 71;
	palette[24].peFlags = PC_NOCOLLAPSE;

	palette[25].peRed   = 144;
	palette[25].peGreen = 99;
	palette[25].peBlue  = 34;
	palette[25].peFlags = PC_NOCOLLAPSE;

	palette[26].peRed   = 172;
	palette[26].peGreen = 102;
	palette[26].peBlue  = 102;
	palette[26].peFlags = PC_NOCOLLAPSE;

	palette[27].peRed   = 212;
	palette[27].peGreen = 214;
	palette[27].peBlue  = 12;
	palette[27].peFlags = PC_NOCOLLAPSE;

	palette[28].peRed   = 253;
	palette[28].peGreen = 254;
	palette[28].peBlue  = 160;
	palette[28].peFlags = PC_NOCOLLAPSE;

	palette[29].peRed   = 249;
	palette[29].peGreen = 226;
	palette[29].peBlue  = 31;
	palette[29].peFlags = PC_NOCOLLAPSE;

	palette[30].peRed   = 149;
	palette[30].peGreen = 127;
	palette[30].peBlue  = 49;
	palette[30].peFlags = PC_NOCOLLAPSE;

	palette[31].peRed   = 213;
	palette[31].peGreen = 161;
	palette[31].peBlue  = 3;
	palette[31].peFlags = PC_NOCOLLAPSE;
	
	palette[32].peRed   = 0;
	palette[32].peGreen = 192;
	palette[32].peBlue  = 192;
	palette[32].peFlags = PC_NOCOLLAPSE;

	palette[33].peRed   = 63;
	palette[33].peGreen = 255;
	palette[33].peBlue  = 253;
	palette[33].peFlags = PC_NOCOLLAPSE;

	palette[34].peRed   = 187;
	palette[34].peGreen = 251;
	palette[34].peBlue  = 250;
	palette[34].peFlags = PC_NOCOLLAPSE;

	palette[35].peRed   = 6;
	palette[35].peGreen = 110;
	palette[35].peBlue  = 109;
	palette[35].peFlags = PC_NOCOLLAPSE;

	palette[36].peRed   = 231;
	palette[36].peGreen = 177;
	palette[36].peBlue  = 177;
	palette[36].peFlags = PC_NOCOLLAPSE;

	palette[37].peRed   = 222;
	palette[37].peGreen = 186;
	palette[37].peBlue  = 186;
	palette[37].peFlags = PC_NOCOLLAPSE;

	palette[38].peRed   = 165;
	palette[38].peGreen = 163;
	palette[38].peBlue  = 163;
	palette[38].peFlags = PC_NOCOLLAPSE;

	palette[39].peRed   = 231;
	palette[39].peGreen = 227;
	palette[39].peBlue  = 227;
	palette[39].peFlags = PC_NOCOLLAPSE;

	palette[40].peRed   = 105;
	palette[40].peGreen = 101;
	palette[40].peBlue  = 101;
	palette[40].peFlags = PC_NOCOLLAPSE;

	palette[41].peRed   = 208;
	palette[41].peGreen = 5;
	palette[41].peBlue  = 221;
	palette[41].peFlags = PC_NOCOLLAPSE;
	
	palette[42].peRed   = 170;
	palette[42].peGreen = 7;
	palette[42].peBlue  = 185;
	palette[42].peFlags = PC_NOCOLLAPSE;

	palette[43].peRed   = 220;
	palette[43].peGreen = 156;
	palette[43].peBlue  = 234;
	palette[43].peFlags = PC_NOCOLLAPSE;

	palette[44].peRed   = 0;
	palette[44].peGreen = 0;
	palette[44].peBlue  = 64;
	palette[44].peFlags = PC_NOCOLLAPSE;

	palette[45].peRed   = 2;
	palette[45].peGreen = 5;
	palette[45].peBlue  = 114;
	palette[45].peFlags = PC_NOCOLLAPSE;

	palette[46].peRed   = 2;
	palette[46].peGreen = 6;
	palette[46].peBlue  = 166;
	palette[46].peFlags = PC_NOCOLLAPSE;

	palette[47].peRed   = 94;
	palette[47].peGreen = 1;
	palette[47].peBlue  = 95;
	palette[47].peFlags = PC_NOCOLLAPSE;

	palette[48].peRed   = 128;
	palette[48].peGreen = 0;
	palette[48].peBlue  = 128;
	palette[48].peFlags = PC_NOCOLLAPSE;

	palette[49].peRed   = 0;
	palette[49].peGreen = 64;
	palette[49].peBlue  = 64;
	palette[49].peFlags = PC_NOCOLLAPSE;

	palette[50].peRed   = 11;
	palette[50].peGreen = 115;
	palette[50].peBlue  = 114;
	palette[50].peFlags = PC_NOCOLLAPSE;

	palette[51].peRed   = 39;
	palette[51].peGreen = 143;
	palette[51].peBlue  = 142;
	palette[51].peFlags = PC_NOCOLLAPSE;
	
	palette[52].peRed   = 0;
	palette[52].peGreen = 64;
	palette[52].peBlue  = 0;
	palette[52].peFlags = PC_NOCOLLAPSE;

	palette[53].peRed   = 7;
	palette[53].peGreen = 137;
	palette[53].peBlue  = 7;
	palette[53].peFlags = PC_NOCOLLAPSE;

	palette[54].peRed   = 146;
	palette[54].peGreen = 147;
	palette[54].peBlue  = 45;
	palette[54].peFlags = PC_NOCOLLAPSE;

	palette[55].peRed   = 155;
	palette[55].peGreen = 156;
	palette[55].peBlue  = 90;
	palette[55].peFlags = PC_NOCOLLAPSE;

	palette[56].peRed   = 148;
	palette[56].peGreen = 26;
	palette[56].peBlue  = 26;
	palette[56].peFlags = PC_NOCOLLAPSE;

	palette[57].peRed   = 118;
	palette[57].peGreen = 2;
	palette[57].peBlue  = 2;
	palette[57].peFlags = PC_NOCOLLAPSE;

	palette[58].peRed   = 174;
	palette[58].peGreen = 4;
	palette[58].peBlue  = 4;
	palette[58].peFlags = PC_NOCOLLAPSE;

	palette[59].peRed   = 168;
	palette[59].peGreen = 72;
	palette[59].peBlue  = 72;
	palette[59].peFlags = PC_NOCOLLAPSE;

	palette[60].peRed   = 148;
	palette[60].peGreen = 102;
	palette[60].peBlue  = 102;
	palette[60].peFlags = PC_NOCOLLAPSE;

	palette[61].peRed   = 190;
	palette[61].peGreen = 162;
	palette[61].peBlue  = 162;
	palette[61].peFlags = PC_NOCOLLAPSE;
	
	palette[62].peRed   = 233;
	palette[62].peGreen = 113;
	palette[62].peBlue  = 7;
	palette[62].peFlags = PC_NOCOLLAPSE;

	palette[63].peRed   = 254;
	palette[63].peGreen = 154;
	palette[63].peBlue  = 74;
	palette[63].peFlags = PC_NOCOLLAPSE;

	palette[64].peRed   = 168;
	palette[64].peGreen = 113;
	palette[64].peBlue  = 48;
	palette[64].peFlags = PC_NOCOLLAPSE;

	palette[65].peRed   = 197;
	palette[65].peGreen = 115;
	palette[65].peBlue  = 43;
	palette[65].peFlags = PC_NOCOLLAPSE;

	palette[66].peRed   = 189;
	palette[66].peGreen = 154;
	palette[66].peBlue  = 109;
	palette[66].peFlags = PC_NOCOLLAPSE;

	palette[67].peRed   = 186;
	palette[67].peGreen = 170;
	palette[67].peBlue  = 126;
	palette[67].peFlags = PC_NOCOLLAPSE;

	palette[68].peRed   = 243;
	palette[68].peGreen = 218;
	palette[68].peBlue  = 171;
	palette[68].peFlags = PC_NOCOLLAPSE;

	palette[69].peRed   = 253;
	palette[69].peGreen = 206;
	palette[69].peBlue  = 131;
	palette[69].peFlags = PC_NOCOLLAPSE;

	palette[70].peRed   = 255;
	palette[70].peGreen = 255;
	palette[70].peBlue  = 192;
	palette[70].peFlags = PC_NOCOLLAPSE;

	palette[71].peRed   = 255;
	palette[71].peGreen = 255;
	palette[71].peBlue  = 128;
	palette[71].peFlags = PC_NOCOLLAPSE;
	
	palette[72].peRed   = 192;
	palette[72].peGreen = 192;
	palette[72].peBlue  = 255;
	palette[72].peFlags = PC_NOCOLLAPSE;

	palette[73].peRed   = 255;
	palette[73].peGreen = 208;
	palette[73].peBlue  = 208;
	palette[73].peFlags = PC_NOCOLLAPSE;

	palette[74].peRed   = 109;
	palette[74].peGreen = 112;
	palette[74].peBlue  = 227;
	palette[74].peFlags = PC_NOCOLLAPSE;

	palette[75].peRed   = 56;
	palette[75].peGreen = 118;
	palette[75].peBlue  = 111;
	palette[75].peFlags = PC_NOCOLLAPSE;

	palette[76].peRed   = 149;
	palette[76].peGreen = 92;
	palette[76].peBlue  = 25;
	palette[76].peFlags = PC_NOCOLLAPSE;

	palette[77].peRed   = 253;
	palette[77].peGreen = 196;
	palette[77].peBlue  = 41;
	palette[77].peFlags = PC_NOCOLLAPSE;

	palette[78].peRed   = 253;
	palette[78].peGreen = 216;
	palette[78].peBlue  = 89;
	palette[78].peFlags = PC_NOCOLLAPSE;

	palette[79].peRed   = 253;
	palette[79].peGreen = 79;
	palette[79].peBlue  = 79;
	palette[79].peFlags = PC_NOCOLLAPSE;

	palette[80].peRed   = 255;
	palette[80].peGreen = 255;
	palette[80].peBlue  = 255;
	palette[80].peFlags = PC_NOCOLLAPSE;

	// now fill in entry 0 and 255 with black and white
	palette[0].peRed   = 0;
	palette[0].peGreen = 0;
	palette[0].peBlue  = 0;
	palette[0].peFlags = PC_NOCOLLAPSE;

	palette[255].peRed   = 255;
	palette[255].peGreen = 255;
	palette[255].peBlue  = 255;
	palette[255].peFlags = PC_NOCOLLAPSE;

	// create the palette object
	if (FAILED(lpdd7->lpVtbl->CreatePalette(lpdd7, DDPCAPS_8BIT | DDPCAPS_ALLOW256, 
                                palette, &lpddpal, NULL)))
		return(0);

	// finally attach the palette to the primary surface
	if (FAILED(lpddsprimary->lpVtbl->SetPalette(lpddsprimary, lpddpal)))
		return(0);
	///////DirectInput Stuff/////
//added & before IID_IDirectDraw7
	// first create the direct input object
	if (FAILED(DirectInputCreateEx(hinstance_app,DIRECTINPUT_VERSION,
					    &IID_IDirectInput7, (void**)&lpdi,NULL)))
		return(0);
	// create a mouse device  /////////////////////////////////////
	if (FAILED(lpdi->lpVtbl->CreateDeviceEx(lpdi, &GUID_SysMouse, &IID_IDirectInputDevice7,
						(void**)&lpdimouse, NULL)))
		return(0);

	// set cooperation level
	if(FAILED(lpdimouse->lpVtbl->SetCooperativeLevel(lpdimouse, main_window_handle, 
						DISCL_NONEXCLUSIVE | DISCL_BACKGROUND)))
		return(0);

	// set data format
	if(FAILED(lpdimouse->lpVtbl->SetDataFormat(lpdimouse, &c_dfDIMouse)))
		return(0);

	// acquire the mouse
	lpdimouse->lpVtbl->Acquire(lpdimouse);
/////////////////////////////////////
	// Create a DI7 object for the keyboard
   	if(FAILED(lpdi->lpVtbl->CreateDeviceEx(lpdi, &GUID_SysKeyboard, &IID_IDirectInputDevice7, (void**)&lpdikey, NULL))) 
		return(0);   

    // Set the data format for the keyboard
    if(FAILED(lpdikey->lpVtbl->SetDataFormat(lpdikey, &c_dfDIKeyboard)))
		return(0);

    // Set the cooperative level for the keyboard
    if(FAILED(lpdikey->lpVtbl->SetCooperativeLevel(lpdikey, main_window_handle, DISCL_FOREGROUND |
                                                       DISCL_NONEXCLUSIVE)))
		return(0);

    // Acquire the keyboard
    lpdikey->lpVtbl->Acquire(lpdikey);

/////////////////////////////////////////////////////////////////

	// set the global mouse position
	mouse_x = SCREEN_WIDTH/2;
	mouse_y = SCREEN_HEIGHT/2;

	// clear the drawing surface
	DDraw_Fill_Surface(lpddsprimary, 0);
	DDraw_Fill_Surface(lpddsback, 0);
	
	return(1);
}
/////////////////////////////////////////////////////////////////////////
int Game_Shutdown(void)
{
	// first unacquire the keyboard
	lpdikey->lpVtbl->Unacquire(lpdikey);
	// now release the keyboard
	lpdikey->lpVtbl->Release(lpdikey);
	lpdikey = NULL;

	// then unacquire the mouse
	lpdimouse->lpVtbl->Unacquire(lpdimouse);
	// now release the mouse
	lpdimouse->lpVtbl->Release(lpdimouse);
	lpdimouse = NULL;
	// release directinput
	lpdi->lpVtbl->Release(lpdi);
	lpdi = NULL;

	// first the palette
	if (lpddpal)
		{
		lpddpal->lpVtbl->Release(lpddpal);
		lpddpal = NULL;
		} // end if

	//then the backbuffer
	if(lpddsback)
		{
		lpddsback->lpVtbl->Release(lpddsback);
		lpddsback = NULL;
		}
	// now the primary surface
	if (lpddsprimary)
		{
		lpddsprimary->lpVtbl->Release(lpddsprimary);
		lpddsprimary = NULL;
		} // end if

	//Return Display mode 
	if (FAILED(lpdd7->lpVtbl->RestoreDisplayMode(lpdd7)))
		return(0);

	// now blow away the IDirectDraw4 interface
	if (lpdd7)
		{
		lpdd7->lpVtbl->Release(lpdd7);
		lpdd7 = NULL;
		} // end if
	
	return(1);
}
